@@ -2,7 +2,7 @@
  * Copyright (C) 2009-2011 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 3 as 
+ * it under the terms of the GNU General Public License version 3 as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -30,6 +30,7 @@
 
 static void test_allocation          (void);
 static void test_desktop_file        (void);
+static void test_desktop_no_icon     (void);
 static void test_urgent              (void);
 static void test_active              (void);
 static void test_get_xids            (void);
@@ -53,6 +54,7 @@ test_application_create_suite (GDBusConnection *connection)
 
   g_test_add_func (DOMAIN"/Allocation", test_allocation);
   g_test_add_func (DOMAIN"/DesktopFile", test_desktop_file);
+  g_test_add_func (DOMAIN"/DesktopFile/NoIcon", test_desktop_no_icon);
   g_test_add_func (DOMAIN"/ManagesXid", test_manages_xid);
   g_test_add_func (DOMAIN"/Xids", test_get_xids);
   g_test_add_func (DOMAIN"/Events/Active", test_active);
@@ -101,6 +103,19 @@ test_desktop_file (void)
   g_object_unref (application);
 }
 
+static void
+test_desktop_no_icon (void)
+{
+  BamfApplication    *application;
+  const char no_icon_desktop[] = TESTDIR"/bamfdaemon/data/no-icon.desktop";
+
+  application = bamf_application_new_from_desktop_file (no_icon_desktop);
+  g_assert (g_strcmp0 (bamf_application_get_desktop_file (application), no_icon_desktop) == 0);
+
+  g_assert (g_strcmp0(bamf_view_get_icon(BAMF_VIEW(application)), "application-default-icon") == 0);
+  g_object_unref (application);
+}
+
 static void
 on_urgent_changed (BamfApplication *application, gboolean result, gpointer data)
 {
@@ -112,53 +127,53 @@ static void
 test_urgent (void)
 {
   signal_seen = FALSE;
-  
+
   BamfApplication *application;
   BamfWindow *window1, *window2;
   BamfLegacyWindowTest *test1, *test2;
-  
+
   application = bamf_application_new ();
-  
+
   g_signal_connect (G_OBJECT (application), "urgent-changed", (GCallback) on_urgent_changed, NULL);
-  
+
   test1 = bamf_legacy_window_test_new (20, "Window X", "class", "exec");
   test2 = bamf_legacy_window_test_new (20, "Window Y", "class", "exec");
-  
+
   window1 = bamf_window_new (BAMF_LEGACY_WINDOW (test1));
   window2 = bamf_window_new (BAMF_LEGACY_WINDOW (test2));
-  
+
   // Ensure we are not visible with no windows
   g_assert (!bamf_view_is_urgent (BAMF_VIEW (application)));
-  
+
   bamf_view_add_child (BAMF_VIEW (application), BAMF_VIEW (window1));
-  
+
   // Test that when added, we signaled properly
   g_assert (!bamf_view_is_urgent (BAMF_VIEW (application)));
   g_assert (!signal_seen);
-  
+
   bamf_view_remove_child (BAMF_VIEW (application), BAMF_VIEW (window1));
-  
+
   // Test that we unset and signal properly
   g_assert (!bamf_view_is_urgent (BAMF_VIEW (application)));
   g_assert (!signal_seen);
-  
+
   bamf_legacy_window_test_set_attention (test1, TRUE);
   bamf_view_add_child (BAMF_VIEW (application), BAMF_VIEW (window1));
-  
-  // Ensure that when adding a skip-tasklist window, we dont set this to visible 
+
+  // Ensure that when adding a skip-tasklist window, we dont set this to visible
   g_assert (bamf_view_is_urgent (BAMF_VIEW (application)));
   g_assert (signal_seen);
   g_assert (signal_result);
-  
+
   signal_seen = FALSE;
 
   bamf_view_add_child (BAMF_VIEW (application), BAMF_VIEW (window2));
-  
+
   g_assert (bamf_view_is_urgent (BAMF_VIEW (application)));
   g_assert (!signal_seen);
-  
+
   bamf_legacy_window_test_set_attention (test1, FALSE);
-  
+
   g_assert (!bamf_view_is_urgent (BAMF_VIEW (application)));
   g_assert (signal_seen);
   g_assert (!signal_result);
@@ -175,53 +190,53 @@ static void
 test_active (void)
 {
   signal_seen = FALSE;
-  
+
   BamfApplication *application;
   BamfWindow *window1, *window2;
   BamfLegacyWindowTest *test1, *test2;
-  
+
   application = bamf_application_new ();
-  
+
   g_signal_connect (G_OBJECT (application), "active-changed", (GCallback) on_active_changed, NULL);
-  
+
   test1 = bamf_legacy_window_test_new (20, "Window X", "class", "exec");
   test2 = bamf_legacy_window_test_new (20, "Window Y", "class", "exec");
-  
+
   window1 = bamf_window_new (BAMF_LEGACY_WINDOW (test1));
   window2 = bamf_window_new (BAMF_LEGACY_WINDOW (test2));
-  
+
   // Ensure we are not active with no windows
   g_assert (!bamf_view_is_active (BAMF_VIEW (application)));
-  
+
   bamf_view_add_child (BAMF_VIEW (application), BAMF_VIEW (window1));
-  
+
   // Test that when added, we signaled properly
   g_assert (!bamf_view_is_active (BAMF_VIEW (application)));
   g_assert (!signal_seen);
-  
+
   bamf_view_remove_child (BAMF_VIEW (application), BAMF_VIEW (window1));
-  
+
   // Test that we unset and signal properly
   g_assert (!bamf_view_is_active (BAMF_VIEW (application)));
   g_assert (!signal_seen);
-  
+
   bamf_legacy_window_test_set_active (test1, TRUE);
   bamf_view_add_child (BAMF_VIEW (application), BAMF_VIEW (window1));
-  
-  // Ensure that when adding a skip-tasklist window, we dont set this to visible 
+
+  // Ensure that when adding a skip-tasklist window, we dont set this to visible
   g_assert (bamf_view_is_active (BAMF_VIEW (application)));
   g_assert (signal_seen);
   g_assert (signal_result);
-  
+
   signal_seen = FALSE;
 
   bamf_view_add_child (BAMF_VIEW (application), BAMF_VIEW (window2));
-  
+
   g_assert (bamf_view_is_active (BAMF_VIEW (application)));
   g_assert (!signal_seen);
-  
+
   bamf_legacy_window_test_set_active (test1, FALSE);
-  
+
   g_assert (!bamf_view_is_active (BAMF_VIEW (application)));
   g_assert (signal_seen);
   g_assert (!signal_result);
@@ -239,7 +254,7 @@ test_get_xids (void)
   guint32 xid;
 
   application = bamf_application_new ();
-  
+
   lwin1 = bamf_legacy_window_test_new (25, "window1", "class", "exec");
   lwin2 = bamf_legacy_window_test_new (50, "window2", "class", "exec");
   window1 = bamf_window_new (BAMF_LEGACY_WINDOW (lwin1));
@@ -289,7 +304,7 @@ test_get_xids (void)
 
   g_variant_iter_free (xids);
   g_variant_unref (container);
- 
+
   g_object_unref (lwin1);
   g_object_unref (lwin2);
   g_object_unref (window1);
@@ -328,59 +343,59 @@ static void
 test_user_visible (void)
 {
   signal_seen = FALSE;
-  
+
   BamfApplication *application;
   BamfWindow *window1, *window2;
   BamfLegacyWindowTest *test1, *test2;
-  
+
   application = bamf_application_new ();
-  
+
   g_signal_connect (G_OBJECT (application), "user-visible-changed", (GCallback) on_user_visible_changed, NULL);
-  
+
   test1 = bamf_legacy_window_test_new (20, "Window X", "class", "exec");
   test2 = bamf_legacy_window_test_new (20, "Window Y", "class", "exec");
-  
+
   window1 = bamf_window_new (BAMF_LEGACY_WINDOW (test1));
   window2 = bamf_window_new (BAMF_LEGACY_WINDOW (test2));
-  
+
   // Ensure we are not visible with no windows
   g_assert (!bamf_view_user_visible (BAMF_VIEW (application)));
-  
+
   bamf_view_add_child (BAMF_VIEW (application), BAMF_VIEW (window1));
-  
+
   // Test that when added, we signaled properly
   g_assert (bamf_view_user_visible (BAMF_VIEW (application)));
   g_assert (signal_seen);
   g_assert (signal_result);
-  
+
   signal_seen = FALSE;
-  
+
   bamf_view_remove_child (BAMF_VIEW (application), BAMF_VIEW (window1));
-  
+
   // Test that we unset and signal properly
   g_assert (!bamf_view_user_visible (BAMF_VIEW (application)));
   g_assert (signal_seen);
   g_assert (!signal_result);
-  
+
   signal_seen = FALSE;
-  
+
   bamf_legacy_window_test_set_skip (test1, TRUE);
   bamf_view_add_child (BAMF_VIEW (application), BAMF_VIEW (window1));
-  
-  // Ensure that when adding a skip-tasklist window, we dont set this to visible 
+
+  // Ensure that when adding a skip-tasklist window, we dont set this to visible
   g_assert (!bamf_view_user_visible (BAMF_VIEW (application)));
   g_assert (!signal_seen);
-  
+
   bamf_view_add_child (BAMF_VIEW (application), BAMF_VIEW (window2));
-  
+
   g_assert (bamf_view_user_visible (BAMF_VIEW (application)));
   g_assert (signal_seen);
   g_assert (signal_result);
-  
+
   signal_seen = FALSE;
-  
+
   bamf_legacy_window_test_set_skip (test2, TRUE);
-  
+
   g_assert (!bamf_view_user_visible (BAMF_VIEW (window1)));
   g_assert (!bamf_view_user_visible (BAMF_VIEW (application)));
   g_assert (signal_seen);
@@ -398,35 +413,35 @@ static void
 test_window_added (void)
 {
   signal_seen = FALSE;
-  
+
   BamfApplication *application;
   BamfWindow *window;
   BamfLegacyWindowTest *test;
   const char *path;
-  
+
   application = bamf_application_new ();
-  
+
   g_signal_connect (G_OBJECT (application), "window-added", (GCallback) on_window_added, NULL);
-  
+
   test = bamf_legacy_window_test_new (20, "Window X", "class", "exec");
   window = bamf_window_new (BAMF_LEGACY_WINDOW (test));
-  
+
   bamf_view_add_child (BAMF_VIEW (application), BAMF_VIEW (window));
-  
+
   // Ensure we dont signal things that are not on the bus
   g_assert (!signal_seen);
-  
+
   bamf_view_remove_child (BAMF_VIEW (application), BAMF_VIEW (window));
-  
+
   path = bamf_view_export_on_bus (BAMF_VIEW (window), gdbus_connection);
-  
+
   bamf_view_add_child (BAMF_VIEW (application), BAMF_VIEW (window));
-  
+
   g_assert (signal_seen);
   g_assert_cmpstr (signal_window, ==, path);
-  
+
   signal_seen = FALSE;
-  
+
   g_object_unref (window);
   g_object_unref (test);
 }
@@ -442,35 +457,35 @@ static void
 test_window_removed (void)
 {
   signal_seen = FALSE;
-  
+
   BamfApplication *application;
   BamfWindow *window;
   BamfLegacyWindowTest *test;
   const char *path;
-  
+
   application = bamf_application_new ();
-  
+
   g_signal_connect (G_OBJECT (application), "window-removed", (GCallback) on_window_removed, NULL);
-  
+
   test = bamf_legacy_window_test_new (20, "Window X", "class", "exec");
   window = bamf_window_new (BAMF_LEGACY_WINDOW (test));
-  
+
   bamf_view_add_child (BAMF_VIEW (application), BAMF_VIEW (window));
   bamf_view_remove_child (BAMF_VIEW (application), BAMF_VIEW (window));
-  
+
   // Ensure we dont signal things that are not on the bus
   g_assert (!signal_seen);
-  
+
   path = bamf_view_export_on_bus (BAMF_VIEW (window), gdbus_connection);
-  
+
   bamf_view_add_child (BAMF_VIEW (application), BAMF_VIEW (window));
   bamf_view_remove_child (BAMF_VIEW (application), BAMF_VIEW (window));
-  
+
   g_assert (signal_seen);
   g_assert (g_strcmp0 (signal_window, path) == 0);
-  
+
   signal_seen = FALSE;
-  
+
   g_object_unref (window);
   g_object_unref (test);
 }
